function [attRc,scatRc] = itReconstrAttScat(n,m,thetas,att,scat,nScat,...
    nIter,indsIn,indsOut,opts)

%%--- INITIALIZATION

%--- parameters for the inverse Radon Transform
filter = opts.IRT.filter;
band = opts.IRT.band;
%--- number of radii for RT, make it even
nrRT = ceil(n*sqrt(2)) + mod(ceil(n*sqrt(2)),2);
%--- offset for the RT data
os = (nrRT-n)/2;

%--- diffusion coefficient
diff = makeDiff(n,opts.diff);

%--- angles for the inverse Radon Transform
angles = 180/pi*(thetas-pi/2);

%--- number of configurations to average over
nAvg = length(indsIn);

%--- allocate
attRc = zeros(n,n,nIter+1);
scatRc = zeros(n,n,nIter+1);
RTs = zeros(nrRT,8*m,nIter+1);

rmin = 1;
rmax = n;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%--- MAIN

calibration = ~strcmp(opts.diff.type,'none') && ...
    (strcmp(opts.diff.rc,'peakCorr') || strcmp(opts.diff.rc,'intCorr'));

%--- if we use measurements without attenuation for the diffusion
%--- correction, we run ballistic measurements
if calibration
    disp('ballistic measurements without attenuation')
    corr = measForRcAtt(n,m,thetas,zeros(n),zeros(n),0,diff,0,...
        opts.input);
    corr = measToRT(n,m,corr,opts.diff,opts.input);
end

%--- measurements for initial guess
disp('measurements for initial guess')
[DAtt,DScat] = measForRcAttScat(n,m,thetas,att,scat,nScat,diff,0,...
    indsIn,indsOut,0,opts.input);

%--- initial guess attenuation
disp('initial guess')

if calibration
    RTs(:,:,1) = [zeros(os,8*m);...
        measToRT(n,m,DAtt,opts.diff,opts.input)-corr; zeros(os,8*m)];
else
    RTs(:,:,1) = [zeros(os,8*m); ...
        measToRT(n,m,DAtt,opts.diff,opts.input); zeros(os,8*m)];
end

attRc(:,:,1) = (n/2)*myIRadon(RTs(:,:,1),filter,angles,band,n);

%--- initial guess scattering
for j=1:nAvg,
    scatRc(:,:,1) = scatRc(:,:,1) + rcScat(n,m,thetas,attRc(:,:,1),...
        DScat(:,:,j),rmin,rmax,indsIn(j),indsOut(j));
end
scatRc(:,:,1) = scatRc(:,:,1)/nAvg;

%--- iterative scheme

for it = 1:nIter,
    disp(['iteration ',num2str(it),' / ',num2str(nIter)])
    %--- measurements for attenuation
    meas = measForRcAtt(n,m,thetas,attRc(:,:,it),scatRc(:,:,it),nScat,...
        diff,1,opts.input);
    %--- new attenuation
    if calibration
        RTs(:,:,it+1) = [zeros(os,8*m); ...
            measToRT(n,m,DAtt-meas,opts.diff,opts.input)-corr;zeros(os,8*m)];
    else
        RTs(:,:,it+1) = [zeros(os,8*m); ...
            measToRT(n,m,DAtt-meas,opts.diff,opts.input);zeros(os,8*m)];
    end
    
    attRc(:,:,it+1) = (n/2)*myIRadon(RTs(:,:,it+1),filter,angles,band,n);
    
    %--- precomputation of the new rotated attenuation
    attRot = rotateImages(attRc(:,:,it+1),thetas(1:2*m));
    
    for j=1:nAvg,
        %--- measurements for scattering
        MScat = measForRcScat(n,m,thetas,attRot,scatRc(:,:,it),nScat,...
            diff,indsIn(j),indsOut(j),rmin,rmax,1,opts.input);
        %--- new reconstruction scattering
        scatRcTemp = scatRc(:,:,1) - rcScat(n,m,thetas,attRc(:,:,it+1),...
            MScat,rmin,rmax,indsIn(j),indsOut(j));
        scatRc(:,:,it+1) = scatRc(:,:,it+1) + scatRcTemp;
    end
    scatRc(:,:,it+1) = scatRc(:,:,it+1)/nAvg;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%--- DISPLAY
xs = -1+1/n:2/n:1-1/n;
for it = 1:nIter+1,
    %--- attenuation-related plots
    figure(20); subplot(2,nIter+1,it); dispmat(attRc(:,:,it))
    if it>=2, 
        subplot(2,nIter+1,nIter+1+it); dispmat(attRc(:,:,it)-attRc(:,:,it-1));
    end
    figure(21); subplot(1,nIter+1,it); dispmat(RTs(:,:,it))
    figure(22); subplot(1,nIter+1,it); dispmat(attRc(:,:,it)-att)
    figure(23);
    subplot(1,2,1); plot(xs,attRc(n/2,:,it)); hold all
    subplot(1,2,2); plot(xs,attRc(ceil(n/3),:,it)); hold all
    
    
    %--- scattering-related plots
    figure(30); subplot(2,nIter+1,it); dispmat(scatRc(:,:,it))
    if it>=2, 
        subplot(2,nIter+1,nIter+1+it); dispmat(scatRc(:,:,it)-scatRc(:,:,it-1));
    end
    figure(31); subplot(1,nIter+1,it); dispmat(scatRc(:,:,it)-scat)
    figure(33);
    subplot(1,2,1); plot(xs,scatRc(n/2,:,it)); hold all
    subplot(1,2,2); plot(xs,scatRc(ceil(n/3),:,it)); hold all    
end
figure(23); 
subplot(1,2,1); plot(xs,att(n/2,:)); title('attenuation - cut at n/2')
subplot(1,2,2); plot(xs,att(ceil(n/3),:)); title('attenuation - cut at n/3')
figure(33); 
subplot(1,2,1); plot(xs,scat(n/2,:)); title('scattering - cut at n/2')
subplot(1,2,2); plot(xs,scat(ceil(n/3),:)); title('scattering - cut at n/3')

end
%----------------------------------------------------------------------

%======================================================================
function [MAtt,MScat] = measForRcAttScat(n,m,thetas,att,scat,...
    nScat,diff,minusBP,indsIn,indsOut,minusSS,optsInput)

MAtt = zeros(n,8*m,n);
MScat = zeros(n,n,length(indsIn));

%--- precomputation of the rotated attenuations
attRot = rotateImages(att,thetas(1:2*m));

%%--- forward code
for th = 1:8*m,
    disp(['theta = ',num2str(th),' / ',num2str(8*m)]);
    tic
    if any(indsIn==th)
        numThOut = find(indsIn==th);
        thOut = indsOut(numThOut);
        parfor indR = 1:n,
            %--- compute the measurements associated with a pulse(indR,th)
            meas = measurements(n,m,thetas,attRot,scat,diff, ...
                makeInput('gaussian',n,m,indR,indIn,1.5/n),nScat,[th,thOut]);
            
            %--- taking out the ballistic part or not
            if minusBP
                MAtt(indR,th,:) = meas(:,th,nScat+1)-meas(:,th,1);
            else
                MAtt(indR,th,:) = meas(:,th,nScat+1);
            end
            
            %--- taking out the single scattering or not
            if minusSS
                MScat(:,indR,numThOut) = meas(:,thOut,nScat+1)-meas(:,thOut,2);
            else
                MScat(:,indR,numThOut) = meas(:,thOut,nScat+1);
            end
        end
    else %--- only measurements for attenuation
        parfor indR = 1:n,
            %--- compute the measurements associated with a pulse(indR,th)
            meas = measurements(n,m,thetas,attRot,scat,diff, ...
                makeInput(n,m,indR,indIn,optsInput),nScat,th);
            
            %--- taking out the ballistic part or not
            if minusBP
                MAtt(indR,th,:) = meas(:,th,nScat+1)-meas(:,th,1);
            else
                MAtt(indR,th,:) = meas(:,th,nScat+1);
            end
            
        end
    end
    toc
end

end
%----------------------------------------------------------------------
